Command

Struct Command 

pub struct Command { /* private fields */ }
Process builder, providing fine-grained control over how a new process should be spawned.

A command is created via Container::command. This struct is similar to std::process::Command.

Implementations

Source

impl Command

pub fn arg(&mut self, arg: &str) -> &mut Self

Adds an argument to pass to the program.

pub fn args<I, S>(&mut self, args: I) -> &mut Self
where I: IntoIterator<Item = S>, S: AsRef<str>,

Adds multiple arguments to pass to the program.

pub fn env(&mut self, key: &str, val: &str) -> &mut Self

Inserts or updates an explicit environment variable mapping.

pub fn envs<I, K, V>(&mut self, vars: I) -> &mut Self
where I: IntoIterator<Item = (K, V)>, K: AsRef<str>, V: AsRef<str>,

Inserts or updates multiple explicit environment variable mappings.

pub fn current_dir<P: AsRef<Path>>(&mut self, dir: P) -> &mut Self

Sets the working directory for the child process.

pub fn wait_timeout(&mut self, timeout: u64) -> &mut Self

Sets the number of seconds to wait for the child process to terminate.

pub fn stdin(&mut self, cfg: Stdio) -> &mut Self

Configuration for the child process’s standard input (stdin) handle.

Defaults to inherit when used with spawn or status, and defaults to piped when used with output.

pub fn stdout(&mut self, cfg: Stdio) -> &mut Self

Configuration for the child process’s standard output (stdout) handle.

Defaults to inherit when used with spawn or status, and defaults to piped when used with output.

pub fn stderr(&mut self, cfg: Stdio) -> &mut Self

Configuration for the child process’s standard error (stderr) handle.

Defaults to inherit when used with spawn or status, and defaults to piped when used with output.

pub fn spawn(&mut self) -> Result<Child>

Executes the command as a child process, returning a handle to it.

pub fn status(&mut self) -> Result<ExitStatus>

Executes a command as a child process, waiting for it to finish and collecting its status.

pub fn output(&mut self) -> Result<Output>

Executes the command as a child process, waiting for it to finish and collecting all of its output.

pub fn get_program(&self) -> &str

Returns the path to the program.

pub fn get_args(&self) -> Vec<String>

Returns the arguments that will be passed to the program.

pub fn get_envs(&self) -> HashMap<String, String>

Returns the environment variables explicitly set for the child process.

pub fn get_current_dir(&self) -> Option<&Path>

Returns the working directory for the child process.